@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2020-2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2020-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -27,8 +27,6 @@
 #include <string>
 #include <algorithm>
 #include <atomic>
-#include <thread>
-#include <mutex>
 #include <variant>
 #include <functional>
 
@@ -39,6 +37,7 @@
 
 #include "mu-maildir.hh"
 #include "mu-query.hh"
+#include "mu-query-parser.hh"
 #include "mu-store.hh"
 
 #include "utils/mu-utils.hh"
@@ -122,7 +121,6 @@ private:
 };
 
 
-
 /// @brief object to manage the server-context for all commands.
 struct Server::Private {
 	Private(Store& store, const Server::Options& opts, Output output)
@@ -134,8 +132,6 @@ struct Server::Private {
 
 	~Private() {
 		indexer().stop();
-		if (index_thread_.joinable())
-			index_thread_.join();
 		if (!tmp_dir_.empty())
 			remove_directory(tmp_dir_);
 	}
@@ -149,6 +145,7 @@ struct Server::Private {
 	Store&            store() { return store_; }
 	const Store&      store() const { return store_; }
 	Indexer&          indexer() { return store().indexer(); }
+	void              do_index(const Indexer::Config& conf);
 	//CommandMap&       command_map() const { return command_map_; }
 
 	//
@@ -213,7 +210,6 @@ private:
 	Server::Output		output_;
 	const CommandHandler	command_handler_;
 	std::atomic<bool>       keep_going_{};
-	std::thread		index_thread_;
 	std::string		tmp_dir_;
 };
 
@@ -280,7 +276,6 @@ msg_sexp_str(const Message& msg, Store::Id docid, const Option<QueryMatch&> qm)
 	return sexpstr;
 }
 
-
 CommandHandler::CommandInfoMap
 Server::Private::make_command_map()
 {
@@ -409,8 +404,11 @@ Server::Private::make_command_map()
 	    "remove",
 	    CommandInfo{
 		ArgMap{{":docid",
-			ArgInfo{Type::Number, true, "document-id for the message to remove"}}},
-		"remove a message from filesystem and database",
+			ArgInfo{Type::Number, false, "document-id for the message to remove"}},
+			{":path",
+			ArgInfo{Type::String, false, "document-id for the message to remove"}}
+		},
+		"remove a message from filesystem and database, using either :docid or :path",
 		[&](const auto& params) { remove_handler(params); }});
 
 	cmap.emplace(
@@ -458,7 +456,17 @@ Server::Private::invoke(const std::string& expr) noexcept
 		keep_going_ = false;
 	} catch (const std::runtime_error& re) {
 		output_sexp(make_error(Error::Code::Internal,
-				       mu_format("caught exception: {}", re.what())));
+				       mu_format("caught runtime exception: {}",
+						 re.what())));
+		keep_going_ = false;
+	} catch (const std::out_of_range& oore) {
+		output_sexp(make_error(Error::Code::Internal,
+				       mu_format("caught out-of-range exception: {}",
+						 oore.what())));
+		keep_going_ = false;
+	} catch (const std::exception& e) {
+		output_sexp(make_error(Error::Code::Internal,
+				       mu_format(" exception: {}", e.what())));
 		keep_going_ = false;
 	} catch (...) {
 		output_sexp(make_error(Error::Code::Internal,
@@ -557,7 +565,6 @@ Server::Private::data_handler(const Command& cmd)
 			"invalid request type '{}'", request_type);
 }
 
-
 /*
  * creating a message object just to get a path seems a bit excessive maybe
  * mu_store_get_path could be added if this turns out to be a problem
@@ -633,45 +640,112 @@ Server::Private::output_results(const QueryResults& qres, size_t batch_size) con
 	return n;
 }
 
+struct FindProps {
+	std::string query;
+	int batch_size{200};
+	int maxnum{-1};
+	Field::Id sort_field_id{Field::Id::Date};
+	QueryFlags flags{QueryFlags::SkipUnreadable};
+};
+// XXX: once we move to C++20, use designated initializers
 
-void
-Server::Private::find_handler(const Command& cmd)
+
+static const std::pair<QueryFlags, std::string> flags_props[] = {
+	{ QueryFlags::Descending, ":descending" },
+	{ QueryFlags::SkipDuplicates, ":skip-dups" },
+	{ QueryFlags::IncludeRelated, ":include-related" },
+	{ QueryFlags::Threading, ":threads" }
+};
+
+/**
+ * Determine the find search properties from a command object
+ *
+ * @param cmd the command
+ *
+ * @return structure with find properties
+ */
+static FindProps
+determine_find_props(const Command& cmd)
 {
-	const auto q{cmd.string_arg(":query").value_or("")};
-	const auto threads{cmd.boolean_arg(":threads")};
-	// perhaps let mu4e set this as frame-lines of the appropriate frame.
-	const auto batch_size{cmd.number_arg(":batch-size").value_or(200)};
-	const auto descending{cmd.boolean_arg(":descending")};
-	const auto maxnum{cmd.number_arg(":maxnum").value_or(-1) /*unlimited*/};
-	const auto skip_dups{cmd.boolean_arg(":skip-dups")};
-	const auto include_related{cmd.boolean_arg(":include-related")};
+	FindProps props{};
 
+	props.query = cmd.string_arg(":query").value_or("");
+	props.batch_size = cmd.number_arg(":batch-size").value_or(200);
+	if (props.batch_size < 1)
+		throw Error{Error::Code::InvalidArgument, "invalid batch-size {}", props.batch_size};
+
+	props.maxnum = cmd.number_arg(":maxnum").value_or(-1) /*unlimited*/;
+	if (props.maxnum < -1)
+		throw Error{Error::Code::InvalidArgument, "invalid max-num {}", props.maxnum};
+
+	const auto threads{cmd.boolean_arg(":threads")};
 	// complicated!
-	auto sort_field_id = std::invoke([&]()->Field::Id {
+	props.sort_field_id = std::invoke([&]()->Field::Id {
 		if (const auto arg = cmd.symbol_arg(":sortfield"); !arg)
 			return Field::Id::Date;
 		else if (arg->length() < 2)
-			throw Error{Error::Code::InvalidArgument, "invalid sort field '{}'",
+			throw Error{Error::Code::InvalidArgument, "invalid sort field parameter '{}'",
 				*arg};
 		else if (const auto field{field_from_name(arg->substr(1))}; !field)
+			// Note: mu4e gives us ':date' etc., hence the 'substr'
 			throw Error{Error::Code::InvalidArgument, "invalid sort field '{}'",
 				*arg};
+		else if (!field->is_sortable())
+			throw Error{Error::Code::InvalidArgument, "not a sortable field '{}'",
+				*arg};
+		else if (threads && field->id != Field::Id::Date)
+			throw Error{Error::Code::InvalidArgument,
+				     "with threads, you can only sort by date, not by '{}'", *arg};
 		else
 			return field->id;
 	});
 
-	if (batch_size < 1)
-		throw Error{Error::Code::InvalidArgument, "invalid batch-size {}", batch_size};
+	for (const auto& item: flags_props) {
+		if (cmd.boolean_arg(item.second))
+			props.flags |= item.first;
+	}
 
-	auto qflags{QueryFlags::SkipUnreadable}; // don't show unreadables.
-	if (descending)
-		qflags |= QueryFlags::Descending;
-	if (skip_dups)
-		qflags |= QueryFlags::SkipDuplicates;
-	if (include_related)
-		qflags |= QueryFlags::IncludeRelated;
-	if (threads)
-		qflags |= QueryFlags::Threading;
+	return props;
+}
+
+/**
+ * Determine the result properties from the find results
+ *
+ * The result properties are passed back to the client
+ *
+ * @param props the find properties
+ *
+ * @return the result properties
+ */
+static Sexp
+determine_result_props(const FindProps& props, size_t found)
+{
+	Sexp resprops;
+
+	resprops.put_props(
+		":found", found,
+		":query", props.query,
+		":query-sexp", parse_query(props.query, false/*!expand*/),
+		":query-sexp-expanded", parse_query(props.query, true/*expand*/),
+		":maxnum", props.maxnum,
+		// mu4e expects ':'-prefixed sort-field, for historical reasons.
+		":sort-field", Sexp::Symbol{mu_format(":{}", field_from_id(props.sort_field_id).name)});
+
+	if (props.maxnum == -1)
+		resprops.put_props(":full", Sexp::t_sym);
+
+	for (const auto& item: flags_props) {
+		if (any_of(props.flags & item.first))
+			resprops.put_props(item.second, Sexp::t_sym);
+	}
+
+	return resprops;
+}
+
+void
+Server::Private::find_handler(const Command& cmd)
+{
+	const auto props = determine_find_props(cmd);
 
 	StopWatch sw{mu_format("{} (indexing: {})", __func__,
 			       indexer().is_running() ? "yes" :  "no")};
@@ -679,7 +753,8 @@ Server::Private::find_handler(const Command& cmd)
 	// we need to _lock_ the store while querying (which likely consists of
 	// multiple actual queries) + grabbing the results.
 	std::lock_guard l{store_.lock()};
-	auto qres{store_.run_query(q, sort_field_id, qflags, maxnum)};
+	auto qres{store_.run_query(props.query, props.sort_field_id,
+				   props.flags, props.maxnum)};
 	if (!qres)
 		throw Error(Error::Code::Query, "failed to run query: {}", qres.error().what());
 
@@ -687,9 +762,10 @@ Server::Private::find_handler(const Command& cmd)
 	 * knows it should erase the headers buffer. this will ensure that the
 	 * output of two finds will not be mixed. */
 	output_sexp(Sexp().put_props(":erase", Sexp::t_sym));
-	const auto bsize{static_cast<size_t>(batch_size)};
+	const auto bsize{static_cast<size_t>(props.batch_size)};
 	const auto foundnum = output_results(*qres, bsize);
-	output_sexp(Sexp().put_props(":found", foundnum));
+
+	output_sexp(determine_result_props(props, foundnum));
 }
 
 void
@@ -748,6 +824,20 @@ get_stats(const Indexer::Progress& stats, const std::string& state)
 	return sexp;
 }
 
+void
+Server::Private::do_index(const Indexer::Config& conf)
+{
+	StopWatch sw{"indexing"};
+	indexer().start(conf);
+	while (indexer().is_running()) {
+		std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+		output_sexp(get_stats(indexer().progress(), "running"),
+			    Server::OutputFlags::Flush);
+	}
+	output_sexp(get_stats(indexer().progress(), "complete"),
+		    Server::OutputFlags::Flush);
+}
+
 void
 Server::Private::index_handler(const Command& cmd)
 {
@@ -757,22 +847,10 @@ Server::Private::index_handler(const Command& cmd)
 	// ignore .noupdate with an empty store.
 	conf.ignore_noupdate = store().empty();
 
-	indexer().stop();
-	if (index_thread_.joinable())
-		index_thread_.join();
+	if (indexer().is_running()) // already
+		throw Error{Error::Code::Xapian, "indexer is already running"};
 
-	// start a background track.
-	index_thread_ = std::thread([this, conf = std::move(conf)] {
-		StopWatch sw{"indexing"};
-		indexer().start(conf);
-		while (indexer().is_running()) {
-			std::this_thread::sleep_for(std::chrono::milliseconds(2000));
-			output_sexp(get_stats(indexer().progress(), "running"),
-				    Server::OutputFlags::Flush);
-		}
-		output_sexp(get_stats(indexer().progress(), "complete"),
-			    Server::OutputFlags::Flush);
-	});
+	do_index(conf);
 }
 
 void
@@ -872,10 +950,14 @@ Server::Private::move_docid(Store::Id		docid,
 }
 
 /*
- * 'move' moves a message to a different maildir and/or changes its
- * flags. parameters are *either* a 'docid:' or 'msgid:' pointing to
- * the message, a 'maildir:' for the target maildir, and a 'flags:'
- * parameter for the new flags.
+ * 'move' moves a message to a different maildir and/or changes its flags.
+ * parameters are *either* a 'docid:' or 'msgid:' pointing to the message, a
+ * 'maildir:' for the target maildir, and a 'flags:' parameter for the new
+ * flags.
+ *
+ * With :msgid, this is "opportunistic": it's not an error when the given
+ * message-id does not exist. This is e.g. for the case when tagging possible
+ * related messages.
  */
 void
 Server::Private::move_handler(const Command& cmd)
@@ -886,7 +968,19 @@ Server::Private::move_handler(const Command& cmd)
 	const auto no_view{cmd.boolean_arg(":noupdate")};
 	const auto docids{determine_docids(store_, cmd)};
 
-	if (docids.size() > 1) {
+	if (docids.empty()) {
+		if (!!cmd.string_arg(":msgid")) {
+			// msgid not found: no problem.
+			mu_debug("no move: '{}' not found",
+				 *cmd.string_arg(":msgid"));
+			return;
+		}
+		// however, if we wanted to be move by msgid, it's worth raising
+		// an error.
+			throw Mu::Error{Error::Code::Store,
+				"message not found in store (docid={})",
+				cmd.number_arg(":docid").value_or(0)};
+	} else if (docids.size() > 1) {
 		if (!maildir.empty()) // ie. duplicate message-ids.
 			throw Mu::Error{Error::Code::Store,
 				"cannot move multiple messages at the same time"};
@@ -894,21 +988,22 @@ Server::Private::move_handler(const Command& cmd)
 		for (auto&& docid : docids)
 			move_docid(docid, flagopt, rename, no_view);
 		return;
+	} else {
+		const auto docid{docids.at(0)};
+		auto    msg = store().find_message(docid)
+			.or_else([&]{throw Error{Error::Code::InvalidArgument,
+						"cannot find message {}", docid};}).value();
+
+		/* if maildir was not specified, take the current one */
+		if (maildir.empty())
+			maildir = msg.maildir();
+
+		/* determine the real target flags, which come from the flags-parameter
+		 * we received (ie., flagstr), if any, plus the existing message
+		 * flags. */
+		const auto flags = calculate_message_flags(msg, flagopt);
+		perform_move(docid, msg, maildir, flags, rename, no_view);
 	}
-	const auto docid{docids.at(0)};
-	auto    msg = store().find_message(docid)
-		.or_else([&]{throw Error{Error::Code::InvalidArgument,
-					"cannot find message {}", docid};}).value();
-
-	/* if maildir was not specified, take the current one */
-	if (maildir.empty())
-		maildir = msg.maildir();
-
-	/* determine the real target flags, which come from the flags-parameter
-	 * we received (ie., flagstr), if any, plus the existing message
-	 * flags. */
-	const auto flags = calculate_message_flags(msg, flagopt);
-	perform_move(docid, msg, maildir, flags, rename, no_view);
 }
 
 void
@@ -951,7 +1046,6 @@ Server::Private::queries_handler(const Command& cmd)
 	output_sexp(Sexp(":queries"_sym, std::move(qresults)));
 }
 
-
 void
 Server::Private::quit_handler(const Command& cmd)
 {
@@ -961,8 +1055,18 @@ Server::Private::quit_handler(const Command& cmd)
 void
 Server::Private::remove_handler(const Command& cmd)
 {
-	const auto docid{cmd.number_arg(":docid").value_or(0)};
-	const auto path{path_from_docid(store(), docid)};
+	auto docid_opt{cmd.number_arg(":docid")};
+	auto path_opt{cmd.string_arg(":path")};
+
+	if (!!docid_opt == !!path_opt)
+		throw Error(Error::Code::InvalidArgument,
+			    "must pass precisely one of :docid and :path");
+	std::string path;
+	Store::Id docid{};
+	if (docid = docid_opt.value_or(0); docid != 0)
+		path = path_from_docid(store(), docid);
+	else
+		path = path_opt.value();
 
 	if (::unlink(path.c_str()) != 0 && errno != ENOENT)
 		throw Error(Error::Code::File,
