@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2022-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -339,6 +339,9 @@ sub_find(CLI::App& sub, Options& opts)
 			{ Format::Json,
 			  {"json", "JSON"}
 			},
+			{ Format::Json2,
+			  {"json2", "more idiomatic JSON"}
+			},
 		}};
 
 	sub.add_flag("--threads,-t", opts.find.threads,
@@ -456,11 +459,12 @@ sub_init(CLI::App& sub, Options& opts)
 			return {};
 	});
 
-	sub.add_option("--maildir,-m", opts.init.maildir, "Top of the maildir")
+	sub.add_option("--maildir,-m", opts.init.maildir, "Root maildir")
 		->type_name("<maildir>")
 		->default_val(default_mdir)
-		->transform(ExpandPath, "expand maildir path")
-		->transform(CanonicalizePath, "canonicalize maildir path");
+		->transform(ExpandPath, "expand maildir path");
+	// don't attempt to canonicalize; in bash this breaks together with
+	// expand path.
 	sub.add_option("--my-address", opts.init.my_addresses,
 		       "Personal e-mail address or regexp")
 		->type_name("<address>");
@@ -513,7 +517,6 @@ sub_move(CLI::App& sub, Options& opts)
 	sub.add_option("source", opts.move.src, "Message file to move")
 		->type_name("<message-path>")
 		->transform(ExpandPath, "expand source path")
-		->transform(CanonicalizePath, "canonicalize source path")
 		->required();
 	sub.add_option("destination", opts.move.dest,
 		       "Destination maildir")
